package XML::XSH2::CompletionList;

use strict;
use vars qw(@XSH_COMMANDS @XSH_NOXPATH_COMMANDS);

@XSH_COMMANDS=qw(
.
add
apropos
assign
backups
call
canonical
catalog
cd
change-ns-prefix
change-ns-uri
chdir
chxpath
clone
close
complete-attributes
complete_attributes
copy
count
cp
create
debug
declare-ns
def
define
defs
del
delete
do
doc-info
doc_info
docs
documents
dtd
dup
echo
edit
edit-string
empty-tags
empty_tags
enc
encoding
eval
exec
exit
exp
expr
files
fold
for
foreach
function
get
hash
help
if
ifinclude
include
indent
index
insert
iterate
keep-blanks
keep_blanks
last
lcd
lineno
list
load-ext-dtd
load-xinclude
load-xincludes
load_ext_dtd
load_xinclude
load_xincludes
local
locate
ls
map
move
mv
my
namespaces
new
next
nobackups
nodebug
normalize
open
parser-completes-attributes
parser-expands-entities
parser-expands-xinclude
parser_completes_attributes
parser_expands_entities
parser_expands_xinclude
pedantic-parser
pedantic_parser
perl
prev
print
process
process-xinclude
process-xincludes
process_xinclude
process_xincludes
prune
pwd
query-encoding
query_encoding
quiet
quit
recovering
redo
regfunc
register-function
register-namespace
register-xhtml-namespace
register-xsh-namespace
regns
regns-xhtml
regns-xsh
remove
rename
return
rm
run-mode
run_mode
save
set
set-dtd
set-enc
set-ns
set-standalone
set_dtd
set_filename
settings
skip-dtd
skip_dtd
sort
stream
strip-whitespace
strip_whitespace
switch-to-new-documents
switch_to_new_documents
system
test-mode
test_mode
throw
transform
try
undef
undefine
unfold
unless
unregfunc
unregister-function
unregister-namespace
unregns
validate
validation
var
variables
vars
verbose
version
while
wrap
wrap-span
wrap_span
xadd
xcopy
xcp
xinclude
xincludes
xinsert
xmove
xmv
xpath-axis-completion
xpath-completion
xpath-extensions
xpath_axis_completion
xpath_completion
xpath_extensions
xsl
xslt
xsltproc
xupdate
);

1;
